@@ -1,18 +1,19 @@
 import operator
+from functools import partial
 from numbers import Number
 
 import numpy as np
 import tlz as toolz
 
 from ..base import tokenize, wait
-from ..delayed import delayed
 from ..blockwise import blockwise
+from ..delayed import delayed
 from ..highlevelgraph import HighLevelGraph
-from ..utils import derived_from, apply
-from .core import dotmany, Array, concatenate, from_delayed
+from ..utils import apply, derived_from
+from .core import Array, concatenate, dotmany, from_delayed
 from .creation import eye
 from .random import RandomState
-from .utils import meta_from_array, svd_flip, ones_like_safe
+from .utils import array_safe, meta_from_array, solve_triangular_safe, svd_flip
 
 
 def _cumsum_blocks(it):
@@ -43,7 +44,7 @@ def _wrapped_qr(a):
     """
     # workaround may be removed when numpy stops rejecting edge cases
     if a.shape[0] == 0:
-        return np.zeros((0, 0)), np.zeros((0, a.shape[1]))
+        return np.zeros_like(a, shape=(0, 0)), np.zeros_like(a, shape=(0, a.shape[1]))
     else:
         return np.linalg.qr(a)
 
@@ -652,7 +653,13 @@ def compression_level(n, q, n_oversamples=10, min_subspace_size=20):
 
 
 def compression_matrix(
-    data, q, iterator="none", n_power_iter=0, n_oversamples=10, seed=None, compute=False
+    data,
+    q,
+    iterator="power",
+    n_power_iter=0,
+    n_oversamples=10,
+    seed=None,
+    compute=False,
 ):
     """Randomly sample matrix to find most active subspace
 
@@ -666,12 +673,13 @@ def compression_matrix(
     q: int
         Size of the desired subspace (the actual size will be bigger,
         because of oversampling, see ``da.linalg.compression_level``)
-    iterator: {'none', 'power', 'QR'}, default='none'
+    iterator: {'power', 'QR'}, default='power'
         Define the technique used for iterations to cope with flat
         singular spectra or when the input matrix is very large.
     n_power_iter: int
-        number of power iterations, useful when the singular values of
-        the input matrix decay very slowly.
+        Number of power iterations, useful when the singular values
+        decay slowly. Error decreases exponentially as `n_power_iter`
+        increases. In practice, set `n_power_iter` <= 4.
     n_oversamples: int, default=10
         Number of oversamples used for generating the sampling matrix.
         This value increases the size of the subspace computed, which is more
@@ -693,6 +701,10 @@ def compression_matrix(
     pp. 217-288, June 2011
     https://arxiv.org/abs/0909.4061
     """
+    if iterator not in ["power", "QR"]:
+        raise ValueError(
+            f"Iterator '{iterator}' not valid, must one one of ['power', 'QR']"
+        )
     m, n = data.shape
     comp_level = compression_level(min(m, n), q, n_oversamples=n_oversamples)
     if isinstance(seed, RandomState):
@@ -717,7 +729,7 @@ def compression_matrix(
                 wait(tmp)
             mat_h = data.dot(tmp)
         q, _ = tsqr(mat_h)
-    elif iterator == "QR":
+    else:
         q, _ = tsqr(mat_h)
         for i in range(n_power_iter):
             if compute:
@@ -728,16 +740,14 @@ def compression_matrix(
                 q = q.persist()
                 wait(q)
             q, _ = tsqr(data.dot(q))
-    else:
-        q, _ = tsqr(mat_h)
     return q.T
 
 
 def svd_compressed(
     a,
     k,
-    iterator="none",
-    n_power_iter=1,
+    iterator="power",
+    n_power_iter=0,
     n_oversamples=10,
     seed=None,
     compute=False,
@@ -756,15 +766,18 @@ def svd_compressed(
         Input array
     k: int
         Rank of the desired thin SVD decomposition.
-    iterator: {'none', 'power', 'QR'}, default='none'
+    iterator: {'power', 'QR'}, default='power'
         Define the technique used for iterations to cope with flat
         singular spectra or when the input matrix is very large.
-    n_power_iter: int, default=1
+    n_power_iter: int, default=0
         Number of power iterations, useful when the singular values
-        decay slowly. Error decreases exponentially as n_power_iter
-        increases. In practice, set n_power_iter <= 4.
+        decay slowly. Error decreases exponentially as `n_power_iter`
+        increases. In practice, set `n_power_iter` <= 4.
     n_oversamples: int, default=10
         Number of oversamples used for generating the sampling matrix.
+        This value increases the size of the subspace computed, which is more
+        accurate at the cost of efficiency.  Results are rarely sensitive to this choice
+        though and in practice a value of 10 is very commonly high enough.
     compute : bool
         Whether or not to compute data at each use.
         Recomputing the input while performing several passes reduces memory
@@ -778,7 +791,7 @@ def svd_compressed(
 
     Examples
     --------
-    >>> u, s, vt = svd_compressed(x, 20)  # doctest: +SKIP
+    >>> u, s, v = svd_compressed(x, 20)  # doctest: +SKIP
 
     Returns
     -------
@@ -795,8 +808,6 @@ def svd_compressed(
     pp. 217-288, June 2011
     https://arxiv.org/abs/0909.4061
     """
-    if iterator != "none" and n_power_iter == 0:
-        raise ValueError("Iterators require n_power_iter > 0.\n")
     comp = compression_matrix(
         a,
         k,
@@ -925,7 +936,7 @@ def svd(a, coerce_signs=True):
         m, n = a.shape
         k = min(a.shape)
         mu, ms, mv = np.linalg.svd(
-            ones_like_safe(a._meta, shape=(1, 1), dtype=a._meta.dtype)
+            np.ones_like(a._meta, shape=(1, 1), dtype=a._meta.dtype)
         )
         u, s, v = delayed(np.linalg.svd, nout=3)(a, full_matrices=False)
         u = from_delayed(u, shape=(m, k), meta=mu)
@@ -954,9 +965,7 @@ def svd(a, coerce_signs=True):
 
 
 def _solve_triangular_lower(a, b):
-    import scipy.linalg
-
-    return scipy.linalg.solve_triangular(a, b, lower=True)
+    return solve_triangular_safe(a, b, lower=True)
 
 
 def lu(a):
@@ -975,7 +984,6 @@ def lu(a):
     l:  Array, lower triangular matrix with unit diagonal.
     u:  Array, upper triangular matrix
     """
-
     import scipy.linalg
 
     if a.ndim != 2:
@@ -1116,8 +1124,6 @@ def solve_triangular(a, b, lower=False):
         Solution to the system `a x = b`. Shape of return matches `b`.
     """
 
-    import scipy.linalg
-
     if a.ndim != 2:
         raise ValueError("a must be 2 dimensional")
     if b.ndim <= 2:
@@ -1178,14 +1184,18 @@ def solve_triangular(a, b, lower=False):
                         prevs.append(prev)
                     target = (operator.sub, target, (sum, prevs))
                 dsk[_key(i, j)] = (
-                    scipy.linalg.solve_triangular,
+                    solve_triangular_safe,
                     (a.name, i, i),
                     target,
                 )
 
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=[a, b])
+
+    a_meta = meta_from_array(a)
+    b_meta = meta_from_array(b)
     res = _solve_triangular_lower(
-        np.array([[1, 0], [1, 2]], dtype=a.dtype), np.array([0, 1], dtype=b.dtype)
+        array_safe([[1, 0], [1, 2]], dtype=a.dtype, like=a_meta),
+        array_safe([0, 1], dtype=b.dtype, like=b_meta),
     )
     meta = meta_from_array(a, b.ndim, dtype=res.dtype)
     return Array(graph, name, shape=b.shape, chunks=b.chunks, meta=meta)
@@ -1241,9 +1251,7 @@ def inv(a):
 
 
 def _cholesky_lower(a):
-    import scipy.linalg
-
-    return scipy.linalg.cholesky(a, lower=True)
+    return np.linalg.cholesky(a)
 
 
 def cholesky(a, lower=False):
@@ -1277,7 +1285,6 @@ def _cholesky(a):
     Private function to perform Cholesky decomposition, which returns both
     lower and upper triangulars.
     """
-    import scipy.linalg
 
     if a.ndim != 2:
         raise ValueError("Dimension must be 2 to perform cholesky decomposition")
@@ -1311,7 +1318,10 @@ def _cholesky(a):
     for i in range(vdim):
         for j in range(hdim):
             if i < j:
-                dsk[name, i, j] = (np.zeros, (a.chunks[0][i], a.chunks[1][j]))
+                dsk[name, i, j] = (
+                    partial(np.zeros_like, shape=(a.chunks[0][i], a.chunks[1][j])),
+                    meta_from_array(a),
+                )
                 dsk[name_upper, j, i] = (name, i, j)
             elif i == j:
                 target = (a.name, i, j)
@@ -1341,7 +1351,8 @@ def _cholesky(a):
 
     graph_upper = HighLevelGraph.from_collections(name_upper, dsk, dependencies=[a])
     graph_lower = HighLevelGraph.from_collections(name, dsk, dependencies=[a])
-    cho = scipy.linalg.cholesky(np.array([[1, 2], [2, 5]], dtype=a.dtype))
+    a_meta = meta_from_array(a)
+    cho = np.linalg.cholesky(array_safe([[1, 2], [2, 5]], dtype=a.dtype, like=a_meta))
     meta = meta_from_array(a, dtype=cho.dtype)
 
     lower = Array(graph_lower, name, shape=a.shape, chunks=a.chunks, meta=meta)
@@ -1350,9 +1361,8 @@ def _cholesky(a):
     return lower, upper
 
 
-def _sort_decreasing(x):
-    x[::-1].sort()
-    return x
+def _reverse(x):
+    return x[::-1]
 
 
 def lstsq(a, b):
@@ -1393,9 +1403,9 @@ def lstsq(a, b):
         Singular values of `a`.
     """
     q, r = qr(a)
-    x = solve_triangular(r, q.T.dot(b))
+    x = solve_triangular(r, q.T.conj().dot(b))
     residuals = b - a.dot(x)
-    residuals = (residuals ** 2).sum(axis=0, keepdims=b.ndim == 1)
+    residuals = abs(residuals ** 2).sum(axis=0, keepdims=b.ndim == 1)
 
     token = tokenize(a, b)
 
@@ -1410,20 +1420,15 @@ def lstsq(a, b):
 
     # singular
     sname = "lstsq-singular-" + token
-    rt = r.T
+    rt = r.T.conj()
     sdsk = {
         (sname, 0): (
-            _sort_decreasing,
-            (np.sqrt, (np.linalg.eigvals, (np.dot, (rt.name, 0, 0), (r.name, 0, 0)))),
+            _reverse,
+            (np.sqrt, (np.linalg.eigvalsh, (np.dot, (rt.name, 0, 0), (r.name, 0, 0)))),
         )
     }
     graph = HighLevelGraph.from_collections(sname, sdsk, dependencies=[rt, r])
-    _, _, _, ss = np.linalg.lstsq(
-        np.array([[1, 0], [1, 2]], dtype=a.dtype),
-        np.array([0, 1], dtype=b.dtype),
-        rcond=-1,
-    )
-    meta = meta_from_array(r, 1, dtype=ss.dtype)
+    meta = meta_from_array(residuals, 1)
     s = Array(graph, sname, shape=(r.shape[0],), chunks=r.shape[0], meta=meta)
 
     return x, residuals, rank, s
