@@ -1,43 +1,35 @@
 import builtins
-from collections.abc import Iterable
+import contextlib
 import operator
+from collections.abc import Iterable
 from functools import partial
 from itertools import product, repeat
-from math import factorial, log, ceil, log2
-
-import numpy as np
+from math import ceil, factorial, log, log2
 from numbers import Integral, Number
 
-from tlz import compose, partition_all, get, accumulate, pluck, drop
+import numpy as np
+from tlz import accumulate, compose, drop, get, partition_all, pluck
 
-from . import chunk
-from .core import _concatenate2, Array, handle_out, implements
-from .blockwise import blockwise
-from ..blockwise import lol_tuples
-from .creation import arange, diagonal
-from .utils import full_like_safe, validate_axis, compute_meta, is_arraylike
-from .wrap import zeros, ones
-from .numpy_compat import ma_divide, divide as np_divide
-from ..base import tokenize
-from ..highlevelgraph import HighLevelGraph
-from ..utils import (
-    ignoring,
-    funcname,
-    Dispatch,
-    deepmap,
-    getargspec,
-    derived_from,
-    is_series_like,
-)
 from .. import config
+from ..base import tokenize
+from ..blockwise import lol_tuples
+from ..highlevelgraph import HighLevelGraph
+from ..utils import deepmap, derived_from, funcname, getargspec, is_series_like
+from . import chunk
+from .blockwise import blockwise
+from .core import Array, _concatenate2, handle_out, implements
+from .creation import arange, diagonal
 
-# Generic functions to support chunks of different types
-empty_lookup = Dispatch("empty")
-empty_lookup.register((object, np.ndarray), np.empty)
-empty_lookup.register(np.ma.masked_array, np.ma.empty)
-divide_lookup = Dispatch("divide")
-divide_lookup.register((object, np.ndarray), np_divide)
-divide_lookup.register(np.ma.masked_array, ma_divide)
+# Keep empty_lookup here for backwards compatibility
+from .dispatch import divide_lookup, empty_lookup  # noqa: F401
+from .utils import (
+    asarray_safe,
+    compute_meta,
+    is_arraylike,
+    meta_from_array,
+    validate_axis,
+)
+from .wrap import ones, zeros
 
 
 def divide(a, b, dtype=None):
@@ -335,7 +327,7 @@ def partial_reduce(
     if np.isscalar(meta):
         return Array(graph, name, out_chunks, dtype=dtype)
     else:
-        with ignoring(AttributeError):
+        with contextlib.suppress(AttributeError):
             meta = meta.astype(dtype)
         return Array(graph, name, out_chunks, meta=meta)
 
@@ -451,74 +443,74 @@ def nansum(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None)
     )
 
 
-with ignoring(AttributeError):
+@derived_from(np)
+def nanprod(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None):
+    if dtype is not None:
+        dt = dtype
+    else:
+        dt = getattr(chunk.nansum(np.empty((1,), dtype=a.dtype)), "dtype", object)
+    return reduction(
+        a,
+        chunk.nanprod,
+        chunk.prod,
+        axis=axis,
+        keepdims=keepdims,
+        dtype=dt,
+        split_every=split_every,
+        out=out,
+    )
 
-    @derived_from(np)
-    def nanprod(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None):
-        if dtype is not None:
-            dt = dtype
-        else:
-            dt = getattr(chunk.nansum(np.empty((1,), dtype=a.dtype)), "dtype", object)
-        return reduction(
-            a,
-            chunk.nanprod,
-            chunk.prod,
-            axis=axis,
-            keepdims=keepdims,
-            dtype=dt,
-            split_every=split_every,
-            out=out,
-        )
 
-    @derived_from(np)
-    def nancumsum(x, axis, dtype=None, out=None, *, method="sequential"):
-        """Dask added an additional keyword-only argument ``method``.
+@derived_from(np)
+def nancumsum(x, axis, dtype=None, out=None, *, method="sequential"):
+    """Dask added an additional keyword-only argument ``method``.
 
-        method : {'sequential', 'blelloch'}, optional
-            Choose which method to use to perform the cumsum.  Default is 'sequential'.
+    method : {'sequential', 'blelloch'}, optional
+        Choose which method to use to perform the cumsum.  Default is 'sequential'.
 
-            * 'sequential' performs the cumsum of each prior block before the current block.
-            * 'blelloch' is a work-efficient parallel cumsum.  It exposes parallelism by
-              first taking the sum of each block and combines the sums via a binary tree.
-              This method may be faster or more memory efficient depending on workload,
-              scheduler, and hardware.  More benchmarking is necessary.
-        """
-        return cumreduction(
-            chunk.nancumsum,
-            operator.add,
-            0,
-            x,
-            axis,
-            dtype,
-            out=out,
-            method=method,
-            preop=np.nansum,
-        )
+        * 'sequential' performs the cumsum of each prior block before the current block.
+        * 'blelloch' is a work-efficient parallel cumsum.  It exposes parallelism by
+            first taking the sum of each block and combines the sums via a binary tree.
+            This method may be faster or more memory efficient depending on workload,
+            scheduler, and hardware.  More benchmarking is necessary.
+    """
+    return cumreduction(
+        chunk.nancumsum,
+        operator.add,
+        0,
+        x,
+        axis,
+        dtype,
+        out=out,
+        method=method,
+        preop=np.nansum,
+    )
 
-    @derived_from(np)
-    def nancumprod(x, axis, dtype=None, out=None, *, method="sequential"):
-        """Dask added an additional keyword-only argument ``method``.
 
-        method : {'sequential', 'blelloch'}, optional
-            Choose which method to use to perform the cumprod.  Default is 'sequential'.
+@derived_from(np)
+def nancumprod(x, axis, dtype=None, out=None, *, method="sequential"):
+    """Dask added an additional keyword-only argument ``method``.
 
-            * 'sequential' performs the cumprod of each prior block before the current block.
-            * 'blelloch' is a work-efficient parallel cumprod.  It exposes parallelism by first
-              taking the product of each block and combines the products via a binary tree.
-              This method may be faster or more memory efficient depending on workload,
-              scheduler, and hardware.  More benchmarking is necessary.
-        """
-        return cumreduction(
-            chunk.nancumprod,
-            operator.mul,
-            1,
-            x,
-            axis,
-            dtype,
-            out=out,
-            method=method,
-            preop=np.nanprod,
-        )
+    method : {'sequential', 'blelloch'}, optional
+        Choose which method to use to perform the cumprod.  Default is 'sequential'.
+
+        * 'sequential' performs the cumprod of each prior block before the current block.
+        * 'blelloch' is a work-efficient parallel cumprod.  It exposes parallelism by first
+            taking the product of each block and combines the products via a binary tree.
+            This method may be faster or more memory efficient depending on workload,
+            scheduler, and hardware.  More benchmarking is necessary.
+    """
+    return cumreduction(
+        chunk.nancumprod,
+        operator.mul,
+        1,
+        x,
+        axis,
+        dtype,
+        out=out,
+        method=method,
+        preop=np.nanprod,
+    )
 
 
 @derived_from(np)
@@ -550,7 +542,7 @@ def nanmax(a, axis=None, keepdims=False, split_every=None, out=None):
 
 
 def numel(x, **kwargs):
-    """ A reduction to count the number of elements """
+    """A reduction to count the number of elements"""
 
     if hasattr(x, "mask"):
         return chunk.sum(np.ones_like(x), **kwargs)
@@ -563,7 +555,7 @@ def numel(x, **kwargs):
     if axis is None:
         prod = np.prod(shape, dtype=dtype)
         return (
-            full_like_safe(x, prod, shape=(1,) * len(shape), dtype=dtype)
+            np.full_like(x, prod, shape=(1,) * len(shape), dtype=dtype)
             if keepdims is True
             else prod
         )
@@ -578,11 +570,11 @@ def numel(x, **kwargs):
         )
     else:
         new_shape = tuple(shape[dim] for dim in range(len(shape)) if dim not in axis)
-    return full_like_safe(x, prod, shape=new_shape, dtype=dtype)
+    return np.full_like(x, prod, shape=new_shape, dtype=dtype)
 
 
 def nannumel(x, **kwargs):
-    """ A reduction to count the number of elements """
+    """A reduction to count the number of elements"""
     return chunk.sum(~(np.isnan(x)), **kwargs)
 
 
@@ -633,7 +625,8 @@ def mean_agg(pairs, dtype="f8", axis=None, computing_meta=False, **kwargs):
     totals = deepmap(lambda pair: pair["total"], pairs)
     total = _concatenate2(totals, axes=axis).sum(axis=axis, dtype=dtype, **kwargs)
 
-    return divide(total, n, dtype=dtype)
+    with np.errstate(divide="ignore", invalid="ignore"):
+        return divide(total, n, dtype=dtype)
 
 
 @derived_from(np)
@@ -678,10 +671,6 @@ def nanmean(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None
     )
 
 
-with ignoring(AttributeError):
-    nanmean = derived_from(np)(nanmean)
-
-
 def moment_chunk(
     A, order=2, sum=chunk.sum, numel=numel, dtype="f8", computing_meta=False, **kwargs
 ):
@@ -875,10 +864,6 @@ def nanvar(
     )
 
 
-with ignoring(AttributeError):
-    nanvar = derived_from(np)(nanvar)
-
-
 def _sqrt(a):
     o = np.sqrt(a)
     if isinstance(o, np.ma.masked_array) and not o.shape and o.mask.all():
@@ -938,13 +923,20 @@ def nanstd(
     return result
 
 
-with ignoring(AttributeError):
-    nanstd = derived_from(np)(nanstd)
-
-
 def _arg_combine(data, axis, argfunc, keepdims=False):
-    """ Merge intermediate results from ``arg_*`` functions"""
-    axis = None if len(axis) == data.ndim or data.ndim == 1 else axis[0]
+    """Merge intermediate results from ``arg_*`` functions"""
+    if isinstance(data, dict):
+        # Array type doesn't support structured arrays (e.g., CuPy),
+        # therefore `data` is stored in a `dict`.
+        assert data["vals"].ndim == data["arg"].ndim
+        axis = (
+            None
+            if len(axis) == data["vals"].ndim or data["vals"].ndim == 1
+            else axis[0]
+        )
+    else:
+        axis = None if len(axis) == data.ndim or data.ndim == 1 else axis[0]
+
     vals = data["vals"]
     arg = data["arg"]
     if axis is None:
@@ -983,9 +975,14 @@ def arg_chunk(func, argfunc, x, axis, offset_info):
             fill_value = np.ma.maximum_fill_value(vals)
         vals = np.ma.filled(vals, fill_value)
 
-    result = np.empty(
-        shape=vals.shape, dtype=[("vals", vals.dtype), ("arg", arg.dtype)]
-    )
+    try:
+        result = np.empty_like(
+            vals, shape=vals.shape, dtype=[("vals", vals.dtype), ("arg", arg.dtype)]
+        )
+    except TypeError:
+        # Array type doesn't support structured arrays (e.g., CuPy)
+        result = dict()
+
     result["vals"] = vals
     result["arg"] = arg
     return result
@@ -993,9 +990,15 @@ def arg_chunk(func, argfunc, x, axis, offset_info):
 
 def arg_combine(func, argfunc, data, axis=None, **kwargs):
     arg, vals = _arg_combine(data, axis, argfunc, keepdims=True)
-    result = np.empty(
-        shape=vals.shape, dtype=[("vals", vals.dtype), ("arg", arg.dtype)]
-    )
+
+    try:
+        result = np.empty_like(
+            vals, shape=vals.shape, dtype=[("vals", vals.dtype), ("arg", arg.dtype)]
+        )
+    except TypeError:
+        # Array type doesn't support structured arrays (e.g., CuPy).
+        result = dict()
+
     result["vals"] = vals
     result["arg"] = arg
     return result
@@ -1063,10 +1066,18 @@ def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None, out=None)
         ((name,) + k, (chunk, (old,) + k, axis, off))
         for (k, off) in zip(keys, offset_info)
     )
-    # The dtype of `tmp` doesn't actually matter, just need to provide something
+
+    dtype = np.argmin(asarray_safe([1], like=meta_from_array(x)))
+    meta = None
+    if is_arraylike(dtype):
+        # This case occurs on non-NumPy types (e.g., CuPy), where the returned
+        # value is an ndarray rather than a scalar.
+        meta = dtype
+        dtype = meta.dtype
+
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=[x])
-    tmp = Array(graph, name, chunks, dtype=x.dtype)
-    dtype = np.argmin([1]).dtype
+    tmp = Array(graph, name, chunks, dtype=dtype, meta=meta)
+
     result = _tree_reduce(tmp, agg, axis, False, dtype, split_every, combine)
     return handle_out(out, result)
 
@@ -1194,7 +1205,7 @@ def prefixscan_blelloch(func, preop, binop, x, axis=None, dtype=None, out=None):
     dask array
     """
     if axis is None:
-        x = x.flatten()
+        x = x.flatten().rechunk(chunks=x.npartitions)
         axis = 0
     if dtype is None:
         dtype = getattr(func(np.empty((0,), dtype=x.dtype)), "dtype", object)
@@ -1340,7 +1351,7 @@ def cumreduction(
         )
 
     if axis is None:
-        x = x.flatten()
+        x = x.flatten().rechunk(chunks=x.npartitions)
         axis = 0
     if dtype is None:
         dtype = getattr(func(np.empty((0,), dtype=x.dtype)), "dtype", object)
@@ -1589,7 +1600,7 @@ def trace(a, offset=0, axis1=0, axis2=1, dtype=None):
 @derived_from(np)
 def median(a, axis=None, keepdims=False, out=None):
     """
-    This works by automatically chunking the reduced axes to a single chunk
+    This works by automatically chunking the reduced axes to a single chunk if necessary
     and then calling ``numpy.median`` function across the remaining dimensions
     """
     if axis is None:
@@ -1603,7 +1614,9 @@ def median(a, axis=None, keepdims=False, out=None):
 
     axis = [ax + a.ndim if ax < 0 else ax for ax in axis]
 
-    a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
+    # rechunk if reduced axes are not contained in a single chunk
+    if builtins.any(a.numblocks[ax] > 1 for ax in axis):
+        a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
 
     result = a.map_blocks(
         np.median,
@@ -1636,7 +1649,9 @@ def nanmedian(a, axis=None, keepdims=False, out=None):
 
     axis = [ax + a.ndim if ax < 0 else ax for ax in axis]
 
-    a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
+    # rechunk if reduced axes are not contained in a single chunk
+    if builtins.any(a.numblocks[ax] > 1 for ax in axis):
+        a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
 
     result = a.map_blocks(
         np.nanmedian,
