@@ -1,5 +1,4 @@
 import itertools
-from operator import getitem
 
 import pytest
 from tlz import merge
@@ -7,33 +6,25 @@ from tlz import merge
 np = pytest.importorskip("numpy")
 
 import dask
-from dask import config
 import dask.array as da
+from dask import config
 from dask.array.slicing import (
     _sanitize_index_element,
     _slice_1d,
-    new_blockdim,
-    sanitize_index,
-    slice_array,
-    take,
-    normalize_index,
-    slicing_plan,
-    make_block_sorted_slices,
-    shuffle_slice,
-)
-from dask.array.slicing import (
-    _sanitize_index_element,
-    _slice_1d,
-    new_blockdim,
-    sanitize_index,
-    slice_array,
-    take,
-    normalize_index,
-    slicing_plan,
     cached_cumsum,
+    make_block_sorted_slices,
+    new_blockdim,
+    normalize_index,
+    sanitize_index,
+    shuffle_slice,
+    slice_array,
+    slicing_plan,
+    take,
 )
 from dask.array.utils import assert_eq, same_keys
 
+from ..chunk import getitem
+
 
 def test_slice_1d():
     expected = {0: slice(10, 25, 1), 1: slice(None, None, None), 2: slice(0, 1, 1)}
@@ -1042,7 +1033,7 @@ def test_make_blockwise_sorted_slice():
     np.testing.assert_array_equal(b, index3)
 
 
-@pytest.mark.filterwarnings("ignore")
+@pytest.mark.filterwarnings("ignore:Slicing:dask.array.core.PerformanceWarning")
 @pytest.mark.parametrize(
     "size, chunks", [((100, 2), (50, 2)), ((100, 2), (37, 1)), ((100,), (55,))]
 )
