@@ -1,42 +1,43 @@
-from distutils.version import LooseVersion
-from collections import defaultdict
-
-from collections import OrderedDict
 import copy
 import json
 import pickle
+import threading
 import warnings
-
-import tlz as toolz
+from collections import OrderedDict, defaultdict
 
 import numpy as np
 import pandas as pd
+import tlz as toolz
+from packaging.version import parse as parse_version
 
 try:
     import fastparquet
     from fastparquet import ParquetFile
-    from fastparquet.util import get_file_scheme
-    from fastparquet.util import ex_from_sep, val_to_num, groupby_types
-    from fastparquet.writer import partition_on_columns, make_part_file
+    from fastparquet.util import ex_from_sep, get_file_scheme, groupby_types, val_to_num
+    from fastparquet.writer import make_part_file, partition_on_columns
 except ImportError:
     pass
 
-from .utils import (
-    _parse_pandas_metadata,
-    _normalize_index_columns,
-    _analyze_paths,
-    _flatten_filters,
-    _row_groups_to_parts,
-)
-from ..utils import _meta_from_dtypes
-from ...utils import UNKNOWN_CATEGORIES
+from ....utils import natural_sort_key
 from ...methods import concat
-
+from ...utils import UNKNOWN_CATEGORIES
+from ..utils import _meta_from_dtypes
 
 #########################
 # Fastparquet interface #
 #########################
-from .utils import Engine
+from .utils import (
+    Engine,
+    _flatten_filters,
+    _get_aggregation_depth,
+    _normalize_index_columns,
+    _parse_pandas_metadata,
+    _row_groups_to_parts,
+    _sort_and_analyze_paths,
+)
+
+# Thread lock required to reset row-groups
+_FP_FILE_LOCK = threading.RLock()
 
 
 def _paths_to_cats(paths, file_scheme):
@@ -106,7 +107,7 @@ paths_to_cats = (
 )
 
 
-def _determine_pf_parts(fs, paths, gather_statistics, **kwargs):
+def _determine_pf_parts(fs, paths, gather_statistics, ignore_metadata_file, **kwargs):
     """Determine how to access metadata and break read into ``parts``
 
     This logic is mostly to handle `gather_statistics=False` cases,
@@ -116,22 +117,31 @@ def _determine_pf_parts(fs, paths, gather_statistics, **kwargs):
     """
     parts = []
     if len(paths) > 1:
-        base, fns = _analyze_paths(paths, fs)
+        paths, base, fns = _sort_and_analyze_paths(paths, fs)
+
+        # Check if _metadata is in paths, and
+        # remove it if ignore_metadata_file=True
+        _metadata_exists = "_metadata" in fns
+        if _metadata_exists and ignore_metadata_file:
+            fns.remove("_metadata")
+            paths = [fs.sep.join([base, fn]) for fn in fns]
+            _metadata_exists = False
+
         if gather_statistics is not False:
             # This scans all the files, allowing index/divisions
             # and filtering
-            if "_metadata" not in fns:
-                paths_use = paths
+            if _metadata_exists:
+                paths_use = fs.sep.join([base, "_metadata"])
             else:
-                paths_use = base + fs.sep + "_metadata"
+                paths_use = paths
             pf = ParquetFile(
                 paths_use, open_with=fs.open, sep=fs.sep, **kwargs.get("file", {})
             )
         else:
-            if "_metadata" in fns:
+            if _metadata_exists:
                 # We have a _metadata file, lets use it
                 pf = ParquetFile(
-                    base + fs.sep + "_metadata",
+                    fs.sep.join([base, "_metadata"]),
                     open_with=fs.open,
                     sep=fs.sep,
                     **kwargs.get("file", {}),
@@ -145,13 +155,28 @@ def _determine_pf_parts(fs, paths, gather_statistics, **kwargs):
                 pf.cats = paths_to_cats(fns, scheme)
                 parts = paths.copy()
     elif fs.isdir(paths[0]):
-        # This is a directory, check for _metadata, then _common_metadata
-        paths = fs.glob(paths[0] + fs.sep + "*")
-        base, fns = _analyze_paths(paths, fs)
-        if "_metadata" in fns:
+        # This is a directory.
+        # Check if _metadata and/or _common_metadata files exists
+        base = paths[0]
+        _metadata_exists = _common_metadata_exists = True
+        if not ignore_metadata_file:
+            _metadata_exists = fs.isfile(fs.sep.join([base, "_metadata"]))
+            _common_metadata_exists = fs.isfile(fs.sep.join([base, "_common_metadata"]))
+
+        # Find all files if we are not using a _metadata file
+        if ignore_metadata_file or not _metadata_exists:
+            # For now, we need to discover every file under paths[0]
+            paths, base, fns = _sort_and_analyze_paths(fs.find(base), fs)
+            _common_metadata_exists = "_common_metadata" in fns
+            if "_metadata" in fns:
+                fns.remove("_metadata")
+                paths = [fs.sep.join([base, fn]) for fn in fns]
+            _metadata_exists = False
+
+        if _metadata_exists:
             # Using _metadata file (best-case scenario)
             pf = ParquetFile(
-                base + fs.sep + "_metadata",
+                fs.sep.join([base, "_metadata"]),
                 open_with=fs.open,
                 sep=fs.sep,
                 **kwargs.get("file", {}),
@@ -165,9 +190,9 @@ def _determine_pf_parts(fs, paths, gather_statistics, **kwargs):
         else:
             # Use _common_metadata file if it is available.
             # Otherwise, just use 0th file
-            if "_common_metadata" in fns:
+            if _common_metadata_exists:
                 pf = ParquetFile(
-                    base + fs.sep + "_common_metadata",
+                    fs.sep.join([base, "_common_metadata"]),
                     open_with=fs.open,
                     **kwargs.get("file", {}),
                 )
@@ -185,6 +210,19 @@ def _determine_pf_parts(fs, paths, gather_statistics, **kwargs):
             paths[0], open_with=fs.open, sep=fs.sep, **kwargs.get("file", {})
         )
 
+    # Ensure that there is no overlap between partition columns
+    # and explicit columns in `pf`
+    if pf.cats:
+        _partitions = [p for p in pf.cats if p not in pf.columns]
+        if not _partitions:
+            pf.cats = {}
+        elif len(_partitions) != len(pf.cats):
+            raise ValueError(
+                "No partition-columns should be written in the \n"
+                "file unless they are ALL written in the file.\n"
+                "columns: {} | partitions: {}".format(pf.columns, pf.cats.keys())
+            )
+
     return parts, pf, gather_statistics, base
 
 
@@ -253,13 +291,13 @@ class FastParquetEngine(Engine):
         dtypes = {storage_name_mapping.get(k, k): v for k, v in dtypes.items()}
 
         index_cols = index or ()
-        meta = _meta_from_dtypes(all_columns, dtypes, index_cols, column_index_names)
         if isinstance(index_cols, str):
             index_cols = [index_cols]
-
-        # fastparquet doesn't handle multiindex
-        if len(index_names) > 1:
-            raise ValueError("Cannot read DataFrame with MultiIndex.")
+        for ind in index_cols:
+            if getattr(dtypes.get(ind), "numpy_dtype", None):
+                # index does not support masked types
+                dtypes[ind] = dtypes[ind].numpy_dtype
+        meta = _meta_from_dtypes(all_columns, dtypes, index_cols, column_index_names)
 
         for cat in categories:
             if cat in meta:
@@ -285,16 +323,23 @@ class FastParquetEngine(Engine):
         split_row_groups,
         index_cols,
         filters,
+        chunksize,
+        aggregation_depth,
     ):
-        # Cannot gather_statistics if our `parts` is alreadey a list
-        # of paths, or if we are building a multiindex (for now).
+        # Cannot gather_statistics if our `parts` is already a list
+        # of paths, or if we are building a multi-index (for now).
         # We also don't "need" to gather statistics if we don't
         # want to apply any filters or calculate divisions.
+        if split_row_groups is None:
+            split_row_groups = False
+        _need_aggregation_stats = chunksize or (
+            int(split_row_groups) > 1 and aggregation_depth
+        )
         if (
             isinstance(parts, list) and len(parts) and isinstance(parts[0], str)
         ) or len(index_cols) > 1:
             gather_statistics = False
-        elif filters is None and len(index_cols) == 0:
+        elif not _need_aggregation_stats and filters is None and len(index_cols) == 0:
             gather_statistics = False
 
         # Make sure gather_statistics allows filtering
@@ -327,8 +372,6 @@ class FastParquetEngine(Engine):
         # statistics are needed for part aggregation.
         if gather_statistics is None:
             gather_statistics = bool(stat_col_indices)
-        if split_row_groups is None:
-            split_row_groups = False
 
         return (
             gather_statistics,
@@ -347,12 +390,27 @@ class FastParquetEngine(Engine):
         dtypes,
         base_path,
         paths,
+        chunksize,
+        aggregation_depth,
     ):
         """Organize row-groups by file."""
 
         # Get partitioning metadata
         pqpartitions = pf.info.get("partitions", None)
 
+        # Fastparquet does not use a natural sorting
+        # order for partitioned data. Re-sort by path
+        if (
+            pqpartitions is not None
+            and aggregation_depth
+            and pf.row_groups
+            and pf.row_groups[0].columns[0].file_path
+        ):
+            pf.row_groups = sorted(
+                pf.row_groups,
+                key=lambda x: natural_sort_key(x.columns[0].file_path),
+            )
+
         # Store types specified in pandas metadata
         pandas_type = {}
         if pf.row_groups and pf.pandas_metadata:
@@ -447,7 +505,7 @@ class FastParquetEngine(Engine):
                             cmax = pd.Timestamp(cmax, tz=tz)
                         last = cmax_last.get(name, None)
 
-                        if not filters:
+                        if not (filters or chunksize or aggregation_depth):
                             # Only think about bailing if we don't need
                             # stats for filtering
                             if cmin is None or (last and cmin < last):
@@ -474,7 +532,10 @@ class FastParquetEngine(Engine):
                             cstats += [cmin, cmax]
                         cmax_last[name] = cmax
                     else:
-                        if not filters and column.meta_data.num_values > 0:
+                        if (
+                            not (filters or chunksize or aggregation_depth)
+                            and column.meta_data.num_values > 0
+                        ):
                             # We are collecting statistics for divisions
                             # only (no filters) - Lets bail.
                             gather_statistics = False
@@ -574,6 +635,8 @@ class FastParquetEngine(Engine):
         base_path,
         paths,
         fs,
+        chunksize,
+        aggregation_depth,
     ):
 
         # Organize row-groups by file
@@ -592,12 +655,15 @@ class FastParquetEngine(Engine):
             dtypes,
             base_path,
             paths,
+            chunksize,
+            aggregation_depth,
         )
 
         # Convert organized row-groups to parts
         parts, stats = _row_groups_to_parts(
             gather_statistics,
             split_row_groups,
+            aggregation_depth,
             file_row_groups,
             file_row_group_stats,
             file_row_group_column_stats,
@@ -627,6 +693,8 @@ class FastParquetEngine(Engine):
         categories,
         split_row_groups,
         gather_statistics,
+        chunksize,
+        aggregation_depth,
     ):
 
         # Check if `parts` is just a list of paths
@@ -648,6 +716,8 @@ class FastParquetEngine(Engine):
             split_row_groups,
             index_cols,
             filters,
+            chunksize,
+            aggregation_depth,
         )
 
         # Process row-groups and return `(parts, stats)`
@@ -662,6 +732,8 @@ class FastParquetEngine(Engine):
             base_path,
             paths,
             fs,
+            chunksize,
+            aggregation_depth,
         )
 
     @classmethod
@@ -674,6 +746,9 @@ class FastParquetEngine(Engine):
         gather_statistics=None,
         filters=None,
         split_row_groups=True,
+        chunksize=None,
+        aggregate_files=None,
+        ignore_metadata_file=False,
         **kwargs,
     ):
         # Define the parquet-file (pf) object to use for metadata,
@@ -681,7 +756,11 @@ class FastParquetEngine(Engine):
         # then each part will correspond to a file.  Otherwise, each part will
         # correspond to a row group (populated below).
         parts, pf, gather_statistics, base_path = _determine_pf_parts(
-            fs, paths, gather_statistics, **kwargs
+            fs,
+            paths,
+            gather_statistics,
+            ignore_metadata_file,
+            **kwargs,
         )
 
         # Process metadata to define `meta` and `index_cols`
@@ -694,6 +773,12 @@ class FastParquetEngine(Engine):
             index,
         ) = cls._generate_dd_meta(pf, index, categories)
 
+        # Check the `aggregate_files` setting
+        aggregation_depth = _get_aggregation_depth(
+            aggregate_files,
+            list(pf.cats),
+        )
+
         # Break `pf` into a list of `parts`
         parts, stats = cls._construct_parts(
             fs,
@@ -707,6 +792,8 @@ class FastParquetEngine(Engine):
             categories,
             split_row_groups,
             gather_statistics,
+            chunksize,
+            aggregation_depth,
         )
 
         # Cannot allow `None` in columns if the user has specified index=False
@@ -718,6 +805,7 @@ class FastParquetEngine(Engine):
         # should avoid breaking the API for now.
         if len(parts):
             parts[0]["common_kwargs"] = {"categories": categories_dict or categories}
+            parts[0]["aggregation_depth"] = aggregation_depth
 
         if len(parts) and len(parts[0]["piece"]) == 1:
 
@@ -731,7 +819,11 @@ class FastParquetEngine(Engine):
         return (meta, stats, parts, index)
 
     @classmethod
-    def read_partition(cls, fs, piece, columns, index, categories=(), **kwargs):
+    def multi_support(cls):
+        return cls == FastParquetEngine
+
+    @classmethod
+    def read_partition(cls, fs, pieces, columns, index, categories=(), **kwargs):
 
         null_index_name = False
         if isinstance(index, list):
@@ -751,31 +843,67 @@ class FastParquetEngine(Engine):
         # the desired row_group
         parquet_file = kwargs.pop("parquet_file", None)
 
-        if isinstance(piece, tuple):
-            if isinstance(piece[0], str):
-                # We have a path to read from
+        # Always convert pieces to list
+        if not isinstance(pieces, list):
+            pieces = [pieces]
+
+        sample = pieces[0]
+        if isinstance(sample, tuple):
+            if isinstance(sample[0], str):
+                # We have paths to read from
                 assert parquet_file is None
+
+                row_groups = []
+                rg_offset = 0
                 parquet_file = ParquetFile(
-                    piece[0], open_with=fs.open, sep=fs.sep, **kwargs.get("file", {})
+                    [p[0] for p in pieces],
+                    open_with=fs.open,
+                    sep=fs.sep,
+                    **kwargs.get("file", {}),
                 )
-                rg_indices = piece[1] or list(range(len(parquet_file.row_groups)))
+                for piece in pieces:
+                    _pf = (
+                        parquet_file
+                        if len(pieces) == 1
+                        else ParquetFile(
+                            piece[0],
+                            open_with=fs.open,
+                            sep=fs.sep,
+                            **kwargs.get("file", {}),
+                        )
+                    )
+                    n_local_row_groups = len(_pf.row_groups)
+                    local_rg_indices = piece[1] or list(range(n_local_row_groups))
+                    row_groups += [
+                        parquet_file.row_groups[rg + rg_offset]
+                        for rg in local_rg_indices
+                    ]
+                    rg_offset += n_local_row_groups
+                update_parquet_file = len(row_groups) < len(parquet_file.row_groups)
 
-                # `piece[1]` will contain row-group indices
-                row_groups = [parquet_file.row_groups[rg] for rg in rg_indices]
             elif parquet_file:
-                # `piece[1]` will contain actual row-group objects,
-                # but they may be pickled
-                row_groups = piece[0]
-                if isinstance(row_groups, bytes):
-                    row_groups = pickle.loads(row_groups)
-                parquet_file.fmd.row_groups = row_groups
-                # NOTE: May lose cats after `_set_attrs` call
-                save_cats = parquet_file.cats
-                parquet_file._set_attrs()
-                parquet_file.cats = save_cats
+
+                row_groups = []
+                for piece in pieces:
+                    # `piece[1]` will contain actual row-group objects,
+                    # but they may be pickled
+                    rgs = piece[0]
+                    if isinstance(rgs, bytes):
+                        rgs = pickle.loads(rgs)
+                    row_groups += rgs
+                update_parquet_file = True
+
             else:
                 raise ValueError("Neither path nor ParquetFile detected!")
 
+            if update_parquet_file:
+                with _FP_FILE_LOCK:
+                    parquet_file.fmd.row_groups = row_groups
+                    # NOTE: May lose cats after `_set_attrs` call
+                    save_cats = parquet_file.cats
+                    parquet_file._set_attrs()
+                    parquet_file.cats = save_cats
+
             if null_index_name:
                 if "__index_level_0__" in parquet_file.columns:
                     # See "Handling a None-labeled index" comment above
@@ -786,19 +914,29 @@ class FastParquetEngine(Engine):
                 lambda *args: parquet_file.dtypes
             )  # ugly patch, could be fixed
 
-            # Read necessary row-groups and concatenate
-            dfs = []
-            for row_group in row_groups:
-                dfs.append(
-                    parquet_file.read_row_group_file(
-                        row_group,
-                        columns,
-                        categories,
-                        index=index,
-                        **kwargs.get("read", {}),
-                    )
+            if set(columns).issubset(
+                parquet_file.columns + list(parquet_file.cats.keys())
+            ):
+                # Convert ParquetFile to pandas
+                return parquet_file.to_pandas(
+                    columns=columns,
+                    categories=categories,
+                    index=index,
                 )
-            return concat(dfs, axis=0) if len(dfs) > 1 else dfs[0]
+            else:
+                # Read necessary row-groups and concatenate
+                dfs = []
+                for row_group in row_groups:
+                    dfs.append(
+                        parquet_file.read_row_group_file(
+                            row_group,
+                            columns,
+                            categories,
+                            index=index,
+                            **kwargs.get("read", {}),
+                        )
+                    )
+                return concat(dfs, axis=0) if len(dfs) > 1 else dfs[0]
 
         else:
             # `piece` is NOT a tuple
@@ -898,15 +1036,25 @@ class FastParquetEngine(Engine):
         return_metadata,
         fmd=None,
         compression=None,
+        custom_metadata=None,
         **kwargs,
     ):
+        # Update key/value metadata if necessary
         fmd = copy.copy(fmd)
+        if custom_metadata and fmd is not None:
+            fmd.key_value_metadata.extend(
+                [
+                    fastparquet.parquet_thrift.KeyValue(key=key, value=value)
+                    for key, value in custom_metadata.items()
+                ]
+            )
+
         if not len(df):
             # Write nothing for empty partitions
             rgs = []
         elif partition_on:
             mkdirs = lambda x: fs.mkdirs(x, exist_ok=True)
-            if LooseVersion(fastparquet.__version__) >= "0.1.4":
+            if parse_version(fastparquet.__version__) >= parse_version("0.1.4"):
                 rgs = partition_on_columns(
                     df, partition_on, path, filename, fmd, compression, fs.open, mkdirs
                 )
