@@ -3,10 +3,11 @@ import re
 import pandas as pd
 
 from ....core import flatten
+from ....utils import natural_sort_key
 
 
 class Engine:
-    """ The API necessary to provide a new Parquet reader/writer """
+    """The API necessary to provide a new Parquet reader/writer"""
 
     @classmethod
     def read_metadata(
@@ -17,7 +18,7 @@ class Engine:
         index=None,
         gather_statistics=None,
         filters=None,
-        **kwargs
+        **kwargs,
     ):
         """Gather metadata about a Parquet Dataset to prepare for a read
 
@@ -108,7 +109,7 @@ class Engine:
         partition_on=None,
         ignore_divisions=False,
         division_info=None,
-        **kwargs
+        **kwargs,
     ):
         """Perform engine-specific initialization steps for this dataset
 
@@ -391,6 +392,12 @@ def _normalize_index_columns(user_columns, data_columns, user_index, data_index)
     return column_names, index_names
 
 
+def _sort_and_analyze_paths(file_list, fs, root=False):
+    file_list = sorted(file_list, key=natural_sort_key)
+    base, fns = _analyze_paths(file_list, fs, root=root)
+    return file_list, base, fns
+
+
 def _analyze_paths(file_list, fs, root=False):
     """Consolidate list of file-paths into parquet relative paths
 
@@ -462,14 +469,12 @@ def _analyze_paths(file_list, fs, root=False):
                     break
             basepath = basepath[:j]
         l = len(basepath)
-
     else:
         basepath = _join_path(root).split("/")
         l = len(basepath)
         assert all(
             p[:l] == basepath for p in path_parts_list
         ), "All paths must begin with the given root"
-    l = len(basepath)
     out_list = []
     for path_parts in path_parts_list:
         out_list.append(
@@ -518,6 +523,7 @@ def _aggregate_stats(
             s = {
                 "file_path_0": file_path,
                 "num-rows": df_rgs["num-rows"].sum(),
+                "num-row-groups": df_rgs["num-rows"].count(),
                 "total_byte_size": df_rgs["total_byte_size"].sum(),
                 "columns": [],
             }
@@ -525,6 +531,7 @@ def _aggregate_stats(
             s = {
                 "file_path_0": file_path,
                 "num-rows": file_row_group_stats[0]["num-rows"],
+                "num-row-groups": 1,
                 "total_byte_size": file_row_group_stats[0]["total_byte_size"],
                 "columns": [],
             }
@@ -556,6 +563,7 @@ def _aggregate_stats(
 def _row_groups_to_parts(
     gather_statistics,
     split_row_groups,
+    aggregation_depth,
     file_row_groups,
     file_row_group_stats,
     file_row_group_column_stats,
@@ -571,10 +579,25 @@ def _row_groups_to_parts(
         # Create parts from each file,
         # limiting the number of row_groups in each piece
         split_row_groups = int(split_row_groups)
+        residual = 0
         for filename, row_groups in file_row_groups.items():
             row_group_count = len(row_groups)
-            for i in range(0, row_group_count, split_row_groups):
+            if residual:
+                _rgs = [0] + list(range(residual, row_group_count, split_row_groups))
+            else:
+                _rgs = list(range(residual, row_group_count, split_row_groups))
+
+            for i in _rgs:
+
                 i_end = i + split_row_groups
+                if aggregation_depth is True:
+                    if residual and i == 0:
+                        i_end = residual
+                        residual = 0
+                    _residual = i_end - row_group_count
+                    if _residual > 0:
+                        residual = _residual
+
                 rg_list = row_groups[i:i_end]
 
                 part = make_part_func(
@@ -616,3 +639,36 @@ def _row_groups_to_parts(
                 stats.append(stat)
 
     return parts, stats
+
+
+def _get_aggregation_depth(aggregate_files, partition_names):
+    # Use `aggregate_files` to set `aggregation_depth`
+    #
+    # Note that `partition_names` must be ordered. `True` means that we allow
+    # aggregation of any two files. `False` means that we will never aggregate
+    # files.  If a string is specified, it must be the name of a partition
+    # column, and the "partition depth" of that column will be used for
+    # aggregation.  Note that we always convert the string into the partition
+    # "depth" to simplify the aggregation logic.
+
+    # Summary of output `aggregation_depth` settings:
+    #
+    # True  : Free-for-all aggregation (any two files may be aggregated)
+    # False : No file aggregation allowed
+    # <int> : Allow aggregation within this partition-hierarchy depth
+
+    aggregation_depth = aggregate_files
+    if isinstance(aggregate_files, str):
+        if aggregate_files in partition_names:
+            # aggregate_files corresponds to a partition column. Reset the
+            # value of this variable to reflect the partition "depth" (in the
+            # range of 1 to the total number of partition levels)
+            aggregation_depth = len(partition_names) - partition_names.index(
+                aggregate_files
+            )
+        else:
+            raise ValueError(
+                f"{aggregate_files} is not a recognized directory partition."
+            )
+
+    return aggregation_depth
