@@ -1,7 +1,6 @@
-from io import BytesIO
-import os
 import gzip
-from time import sleep
+import os
+from io import BytesIO
 from unittest import mock
 
 import pytest
@@ -9,24 +8,25 @@ import pytest
 pd = pytest.importorskip("pandas")
 dd = pytest.importorskip("dask.dataframe")
 
+from fsspec.compression import compr
 from tlz import partition_all, valmap
 
 import dask
 import dask.dataframe as dd
-from dask.dataframe._compat import tm
 from dask.base import compute_as_if_collection
-from dask.core import flatten
-from dask.dataframe.io.csv import (
-    text_blocks_to_pandas,
-    pandas_read_text,
-    auto_blocksize,
-    block_mask,
-)
-from dask.dataframe.utils import assert_eq, has_known_categories
 from dask.bytes.core import read_bytes
 from dask.bytes.utils import compress
-from dask.utils import filetexts, filetext, tmpfile, tmpdir
-from fsspec.compression import compr
+from dask.core import flatten
+from dask.dataframe._compat import tm
+from dask.dataframe.io.csv import (
+    _infer_block_size,
+    auto_blocksize,
+    block_mask,
+    pandas_read_text,
+    text_blocks_to_pandas,
+)
+from dask.dataframe.utils import assert_eq, has_known_categories
+from dask.utils import filetext, filetexts, tmpdir, tmpfile
 
 # List of available compression format for test_read_csv_compression
 compression_fmts = [fmt for fmt in compr] + [None]
@@ -124,19 +124,19 @@ comment_header = b"""# some header lines
 # in a data file
 # before any data"""
 
+comment_footer = b"""# some footer lines
+# that may be present
+# at the end of the file"""
+
 csv_units_row = b"str, int, int\n"
 tsv_units_row = csv_units_row.replace(b",", b"\t")
 
 
-# Pandas has deprecated read_table
-read_table_mark = pytest.mark.filterwarnings("ignore:read_table:FutureWarning")
-
-
 csv_and_table = pytest.mark.parametrize(
     "reader,files",
     [
         (pd.read_csv, csv_files),
-        pytest.param(pd.read_table, tsv_files, marks=read_table_mark),
+        (pd.read_table, tsv_files),
         (pd.read_fwf, fwf_files),
     ],
 )
@@ -240,7 +240,6 @@ def test_text_blocks_to_pandas_blocked(reader, files):
     "dd_read,pd_read,files",
     [(dd.read_csv, pd.read_csv, csv_files), (dd.read_table, pd.read_table, tsv_files)],
 )
-@read_table_mark
 def test_skiprows(dd_read, pd_read, files):
     files = {name: comment_header + b"\n" + content for name, content in files.items()}
     skip = len(comment_header.splitlines())
@@ -250,6 +249,21 @@ def test_skiprows(dd_read, pd_read, files):
         assert_eq(df, expected_df, check_dtype=False)
 
 
+@pytest.mark.parametrize(
+    "dd_read,pd_read,files",
+    [(dd.read_csv, pd.read_csv, csv_files), (dd.read_table, pd.read_table, tsv_files)],
+)
+def test_skipfooter(dd_read, pd_read, files):
+    files = {name: content + b"\n" + comment_footer for name, content in files.items()}
+    skip = len(comment_footer.splitlines())
+    with filetexts(files, mode="b"):
+        df = dd_read("2014-01-*.csv", skipfooter=skip, engine="python")
+        expected_df = pd.concat(
+            [pd_read(n, skipfooter=skip, engine="python") for n in sorted(files)]
+        )
+        assert_eq(df, expected_df, check_dtype=False)
+
+
 @pytest.mark.parametrize(
     "dd_read,pd_read,files,units",
     [
@@ -257,7 +271,6 @@ def test_skiprows(dd_read, pd_read, files):
         (dd.read_table, pd.read_table, tsv_files, tsv_units_row),
     ],
 )
-@read_table_mark
 def test_skiprows_as_list(dd_read, pd_read, files, units):
     files = {
         name: (comment_header + b"\n" + content.replace(b"\n", b"\n" + units, 1))
@@ -284,7 +297,6 @@ tsv_blocks = [
 @pytest.mark.parametrize(
     "reader,blocks", [(pd.read_csv, csv_blocks), (pd.read_table, tsv_blocks)]
 )
-@read_table_mark
 def test_enforce_dtypes(reader, blocks):
     head = reader(BytesIO(blocks[0][0]), header=0)
     header = blocks[0][0].split(b"\n")[0] + b"\n"
@@ -296,7 +308,6 @@ def test_enforce_dtypes(reader, blocks):
 @pytest.mark.parametrize(
     "reader,blocks", [(pd.read_csv, csv_blocks), (pd.read_table, tsv_blocks)]
 )
-@read_table_mark
 def test_enforce_columns(reader, blocks):
     # Replace second header with different column name
     blocks = [blocks[0], [blocks[1][0].replace(b"a", b"A"), blocks[1][1]]]
@@ -320,7 +331,6 @@ def test_enforce_columns(reader, blocks):
         (dd.read_table, pd.read_table, tsv_text2, r"\s+"),
     ],
 )
-@read_table_mark
 def test_read_csv(dd_read, pd_read, text, sep):
     with filetext(text) as fn:
         f = dd_read(fn, blocksize=30, lineterminator=os.linesep, sep=sep)
@@ -337,7 +347,6 @@ def test_read_csv(dd_read, pd_read, text, sep):
         (dd.read_table, pd.read_table, tsv_text, [1, 13]),
     ],
 )
-@read_table_mark
 def test_read_csv_large_skiprows(dd_read, pd_read, text, skip):
     names = ["name", "amount"]
     with filetext(text) as fn:
@@ -352,7 +361,6 @@ def test_read_csv_large_skiprows(dd_read, pd_read, text, skip):
         (dd.read_table, pd.read_table, tsv_text, [1, 12]),
     ],
 )
-@read_table_mark
 def test_read_csv_skiprows_only_in_first_partition(dd_read, pd_read, text, skip):
     names = ["name", "amount"]
     with filetext(text) as fn:
@@ -370,7 +378,6 @@ def test_read_csv_skiprows_only_in_first_partition(dd_read, pd_read, text, skip)
     "dd_read,pd_read,files",
     [(dd.read_csv, pd.read_csv, csv_files), (dd.read_table, pd.read_table, tsv_files)],
 )
-@read_table_mark
 def test_read_csv_files(dd_read, pd_read, files):
     with filetexts(files, mode="b"):
         df = dd_read("2014-01-*.csv")
@@ -386,7 +393,6 @@ def test_read_csv_files(dd_read, pd_read, files):
     "dd_read,pd_read,files",
     [(dd.read_csv, pd.read_csv, csv_files), (dd.read_table, pd.read_table, tsv_files)],
 )
-@read_table_mark
 def test_read_csv_files_list(dd_read, pd_read, files):
     with filetexts(files, mode="b"):
         subset = sorted(files)[:2]  # Just first 2
@@ -401,7 +407,6 @@ def test_read_csv_files_list(dd_read, pd_read, files):
 @pytest.mark.parametrize(
     "dd_read,files", [(dd.read_csv, csv_files), (dd.read_table, tsv_files)]
 )
-@read_table_mark
 def test_read_csv_include_path_column(dd_read, files):
     with filetexts(files, mode="b"):
         df = dd_read(
@@ -418,7 +423,6 @@ def test_read_csv_include_path_column(dd_read, files):
 @pytest.mark.parametrize(
     "dd_read,files", [(dd.read_csv, csv_files), (dd.read_table, tsv_files)]
 )
-@read_table_mark
 def test_read_csv_include_path_column_as_str(dd_read, files):
     with filetexts(files, mode="b"):
         df = dd_read(
@@ -435,7 +439,6 @@ def test_read_csv_include_path_column_as_str(dd_read, files):
 @pytest.mark.parametrize(
     "dd_read,files", [(dd.read_csv, csv_files), (dd.read_table, tsv_files)]
 )
-@read_table_mark
 def test_read_csv_include_path_column_with_duplicate_name(dd_read, files):
     with filetexts(files, mode="b"):
         with pytest.raises(ValueError):
@@ -445,7 +448,6 @@ def test_read_csv_include_path_column_with_duplicate_name(dd_read, files):
 @pytest.mark.parametrize(
     "dd_read,files", [(dd.read_csv, csv_files), (dd.read_table, tsv_files)]
 )
-@read_table_mark
 def test_read_csv_include_path_column_is_dtype_category(dd_read, files):
     with filetexts(files, mode="b"):
         df = dd_read("2014-01-*.csv", include_path_column=True)
@@ -461,7 +463,6 @@ def test_read_csv_include_path_column_is_dtype_category(dd_read, files):
 @pytest.mark.parametrize(
     "dd_read,files", [(dd.read_csv, csv_files), (dd.read_table, tsv_files)]
 )
-@read_table_mark
 def test_read_csv_include_path_column_with_multiple_partitions_per_file(dd_read, files):
     with filetexts(files, mode="b"):
         df = dd_read("2014-01-*.csv", blocksize="10B", include_path_column=True)
@@ -509,8 +510,11 @@ def test_read_csv_skiprows_range():
 def test_usecols():
     with filetext(timeseries) as fn:
         df = dd.read_csv(fn, blocksize=30, usecols=["High", "Low"])
+        df_select = df[["High"]]
         expected = pd.read_csv(fn, usecols=["High", "Low"])
+        expected_select = expected[["High"]]
         assert (df.compute().values == expected.values).all()
+        assert (df_select.compute().values == expected_select.values).all()
 
 
 def test_string_blocksize():
@@ -784,6 +788,23 @@ def test_auto_blocksize():
     assert auto_blocksize(5000, 2) == 250
 
 
+def test__infer_block_size(monkeypatch):
+    """
+    psutil returns a total memory of `None` on some systems
+    see https://github.com/dask/dask/pull/7601
+    """
+    psutil = pytest.importorskip("psutil")
+
+    class MockOutput:
+        total = None
+
+    def mock_virtual_memory():
+        return MockOutput
+
+    monkeypatch.setattr(psutil, "virtual_memory", mock_virtual_memory)
+    assert _infer_block_size()
+
+
 def test_auto_blocksize_max64mb():
     blocksize = auto_blocksize(1000000000000, 3)
     assert blocksize == int(64e6)
@@ -864,20 +885,6 @@ def test_csv_with_integer_names():
         assert list(df.columns) == [0, 1]
 
 
-@pytest.mark.slow
-def test_read_csv_of_modified_file_has_different_name():
-    with filetext(csv_text) as fn:
-        sleep(1)
-        a = dd.read_csv(fn)
-        sleep(1)
-        with open(fn, "a") as f:
-            f.write("\nGeorge,700")
-            os.fsync(f)
-        b = dd.read_csv(fn)
-
-        assert sorted(a.dask, key=str) != sorted(b.dask, key=str)
-
-
 def test_late_dtypes():
     text = "numbers,names,more_numbers,integers,dates\n"
     for i in range(1000):
@@ -1070,9 +1077,7 @@ def test_read_csv_with_datetime_index_partitions_n():
         assert_eq(df, ddf)
 
 
-xfail_pandas_100 = pytest.mark.xfail(
-    dd._compat.PANDAS_GT_100, reason="https://github.com/dask/dask/issues/5787"
-)
+xfail_pandas_100 = pytest.mark.xfail(reason="https://github.com/dask/dask/issues/5787")
 
 
 @pytest.mark.parametrize(
@@ -1185,12 +1190,26 @@ def test_robust_column_mismatch():
     k = sorted(files)[-1]
     files[k] = files[k].replace(b"name", b"Name")
     with filetexts(files, mode="b"):
-        ddf = dd.read_csv("2014-01-*.csv")
+        ddf = dd.read_csv(
+            "2014-01-*.csv", header=None, skiprows=1, names=["name", "amount", "id"]
+        )
         df = pd.read_csv("2014-01-01.csv")
         assert (df.columns == ddf.columns).all()
         assert_eq(ddf, ddf)
 
 
+def test_different_columns_are_allowed():
+    files = csv_files.copy()
+    k = sorted(files)[-1]
+    files[k] = files[k].replace(b"name", b"address")
+    with filetexts(files, mode="b"):
+        ddf = dd.read_csv("2014-01-*.csv")
+
+        # since enforce is False, meta doesn't have to match computed
+        assert (ddf.columns == ["name", "amount", "id"]).all()
+        assert (ddf.compute().columns == ["name", "amount", "id", "address"]).all()
+
+
 def test_error_if_sample_is_too_small():
     text = "AAAAA,BBBBB,CCCCC,DDDDD,EEEEE\n1,2,3,4,5\n6,7,8,9,10\n11,12,13,14,15"
     with filetext(text) as fn:
@@ -1631,3 +1650,66 @@ def test_read_csv_groupby_get_group(tmpdir):
     ddfs = ddf1.groupby("foo")
 
     assert_eq(df1, ddfs.get_group(10).compute())
+
+
+def test_csv_getitem_column_order(tmpdir):
+    # See: https://github.com/dask/dask/issues/7759
+
+    path = os.path.join(str(tmpdir), "test.csv")
+    columns = list("abcdefghijklmnopqrstuvwxyz")
+    values = list(range(len(columns)))
+
+    df1 = pd.DataFrame([{c: v for c, v in zip(columns, values)}])
+    df1.to_csv(path)
+
+    # Use disordered and duplicated column selection
+    columns = list("hczzkylaape")
+    df2 = dd.read_csv(path)[columns].head(1)
+    assert_eq(df1[columns], df2)
+
+
+def test_csv_parse_fail(tmpdir):
+    # See GH #7680
+    path = os.path.join(str(tmpdir), "test.csv")
+    data = b'a,b\n1,"hi\n"\n2,"oi\n"\n'
+    expected = pd.read_csv(BytesIO(data))
+    with open(path, "wb") as f:
+        f.write(data)
+    with pytest.raises(ValueError, match="EOF encountered"):
+        dd.read_csv(path, sample=13)
+    df = dd.read_csv(path, sample=13, sample_rows=1)
+    assert_eq(df, expected)
+
+
+def test_csv_name_should_be_different_even_if_head_is_same(tmpdir):
+    # https://github.com/dask/dask/issues/7904
+    import random
+    from shutil import copyfile
+
+    old_csv_path = os.path.join(str(tmpdir), "old.csv")
+    new_csv_path = os.path.join(str(tmpdir), "new_csv")
+
+    # Create random CSV
+    with open(old_csv_path, "w") as f:
+        for _ in range(10):
+            f.write(
+                f"{random.randrange(1, 10**9):09}, {random.randrange(1, 10**9):09}, {random.randrange(1, 10**9):09}\n"
+            )
+
+    copyfile(old_csv_path, new_csv_path)
+
+    # Add three new rows
+    with open(new_csv_path, "a") as f:
+        for _ in range(3):
+            f.write(
+                f"{random.randrange(1, 10**9):09}, {random.randrange(1, 10**9):09}, {random.randrange(1, 10**9):09}\n"
+            )
+
+    new_df = dd.read_csv(
+        new_csv_path, header=None, delimiter=",", dtype=str, blocksize=None
+    )
+    old_df = dd.read_csv(
+        old_csv_path, header=None, delimiter=",", dtype=str, blocksize=None
+    )
+
+    assert new_df.dask.keys() != old_df.dask.keys()
