@@ -1,12 +1,12 @@
-import pandas as pd
 import numpy as np
+import pandas as pd
 from pandas.core.resample import Resampler as pd_Resampler
 
-from ..core import DataFrame, Series
 from ...base import tokenize
-from ...utils import derived_from
 from ...highlevelgraph import HighLevelGraph
+from ...utils import derived_from
 from .. import methods
+from ..core import DataFrame, Series
 
 
 def getnanos(rule):
@@ -90,7 +90,7 @@ def _resample_bin_and_out_divs(divisions, rule, closed="left", label="left"):
     return tuple(map(pd.Timestamp, newdivs)), tuple(map(pd.Timestamp, outdivs))
 
 
-class Resampler(object):
+class Resampler:
     """Class for resampling timeseries data.
 
     This class is commonly encountered when using ``obj.resample(...)`` which
