@@ -1,15 +1,15 @@
 import random
 from bisect import bisect_left
-from distutils.version import LooseVersion
 from itertools import cycle
-from operator import itemgetter, add
+from operator import add, itemgetter
+
+from packaging.version import parse as parse_version
+from tlz import accumulate, groupby, pluck, unique
 
-from ..utils import funcname, import_required, apply
 from ..core import istask
-
+from ..utils import apply, funcname, import_required
 
 _BOKEH_MISSING_MSG = "Diagnostics plots require `bokeh` to be installed"
-_TOOLZ_MISSING_MSG = "Diagnostics plots require `toolz` to be installed"
 
 
 def unquote(expr):
@@ -129,16 +129,15 @@ def get_colors(palette, funcs):
         Iterable of function names
     """
     palettes = import_required("bokeh.palettes", _BOKEH_MISSING_MSG)
-    tz = import_required("tlz", _TOOLZ_MISSING_MSG)
 
-    unique_funcs = list(sorted(tz.unique(funcs)))
+    unique_funcs = sorted(unique(funcs))
     n_funcs = len(unique_funcs)
     palette_lookup = palettes.all_palettes[palette]
     keys = list(sorted(palette_lookup.keys()))
     index = keys[min(bisect_left(keys, n_funcs), len(keys) - 1)]
     palette = palette_lookup[index]
     # Some bokeh palettes repeat colors, we want just the unique set
-    palette = list(tz.unique(palette))
+    palette = list(unique(palette))
     if len(palette) > n_funcs:
         # Consistently shuffle palette - prevents just using low-range
         random.Random(42).shuffle(palette)
@@ -172,18 +171,9 @@ def visualize(profilers, file_path=None, show=True, save=True, mode=None, **kwar
     The completed bokeh plot object.
     """
     bp = import_required("bokeh.plotting", _BOKEH_MISSING_MSG)
-    import bokeh
+    from bokeh.io import state
 
-    if LooseVersion(bokeh.__version__) >= "0.12.10":
-        from bokeh.io import state
-
-        in_notebook = state.curstate().notebook
-    else:
-        from bokeh.io import _state
-
-        in_notebook = _state._notebook
-
-    if not in_notebook:
+    if not state.curstate().notebook:
         file_path = file_path or "profile.html"
         bp.output_file(file_path, mode=mode)
 
@@ -215,13 +205,6 @@ def visualize(profilers, file_path=None, show=True, save=True, mode=None, **kwar
     return p
 
 
-def _get_figure_keywords():
-    bp = import_required("bokeh.plotting", _BOKEH_MISSING_MSG)
-    o = bp.Figure.properties()
-    o.add("tools")
-    return o
-
-
 def plot_tasks(results, dsk, palette="Viridis", label_size=60, **kwargs):
     """Visualize the results of profiling in a bokeh plot.
 
@@ -247,8 +230,6 @@ def plot_tasks(results, dsk, palette="Viridis", label_size=60, **kwargs):
     bp = import_required("bokeh.plotting", _BOKEH_MISSING_MSG)
     from bokeh.models import HoverTool
 
-    tz = import_required("tlz", _TOOLZ_MISSING_MSG)
-
     defaults = dict(
         title="Profile Results",
         tools="hover,save,reset,xwheel_zoom,xpan",
@@ -261,12 +242,12 @@ def plot_tasks(results, dsk, palette="Viridis", label_size=60, **kwargs):
         kwargs["width"] = kwargs.pop("plot_width")
     if "plot_height" in kwargs:
         kwargs["height"] = kwargs.pop("plot_height")
-    defaults.update((k, v) for (k, v) in kwargs.items() if k in _get_figure_keywords())
+    defaults.update(**kwargs)
 
     if results:
         keys, tasks, starts, ends, ids = zip(*results)
 
-        id_group = tz.groupby(itemgetter(4), results)
+        id_group = groupby(itemgetter(4), results)
         timings = dict(
             (k, [i.end_time - i.start_time for i in v]) for (k, v) in id_group.items()
         )
@@ -364,7 +345,13 @@ def plot_resources(results, palette="Viridis", **kwargs):
         kwargs["width"] = kwargs.pop("plot_width")
     if "plot_height" in kwargs:
         kwargs["height"] = kwargs.pop("plot_height")
-    defaults.update((k, v) for (k, v) in kwargs.items() if k in _get_figure_keywords())
+
+    # Drop `label_size` to match `plot_cache` and `plot_tasks` kwargs
+    if "label_size" in kwargs:
+        kwargs.pop("label_size")
+
+    defaults.update(**kwargs)
+
     if results:
         t, mem, cpu = zip(*results)
         left, right = min(t), max(t)
@@ -385,7 +372,7 @@ def plot_resources(results, palette="Viridis", **kwargs):
         line_width=4,
         **{
             "legend_label"
-            if LooseVersion(bokeh.__version__) >= "1.4"
+            if parse_version(bokeh.__version__) >= parse_version("1.4")
             else "legend": "% CPU"
         }
     )
@@ -403,7 +390,7 @@ def plot_resources(results, palette="Viridis", **kwargs):
         line_width=4,
         **{
             "legend_label"
-            if LooseVersion(bokeh.__version__) >= "1.4"
+            if parse_version(bokeh.__version__) >= parse_version("1.4")
             else "legend": "Memory"
         }
     )
@@ -448,8 +435,6 @@ def plot_cache(
     bp = import_required("bokeh.plotting", _BOKEH_MISSING_MSG)
     from bokeh.models import HoverTool
 
-    tz = import_required("tlz", _TOOLZ_MISSING_MSG)
-
     defaults = dict(
         title="Profile Results",
         tools="hover,save,reset,wheel_zoom,xpan",
@@ -462,19 +447,19 @@ def plot_cache(
         kwargs["width"] = kwargs.pop("plot_width")
     if "plot_height" in kwargs:
         kwargs["height"] = kwargs.pop("plot_height")
-    defaults.update((k, v) for (k, v) in kwargs.items() if k in _get_figure_keywords())
+    defaults.update(**kwargs)
 
     if results:
         starts, ends = list(zip(*results))[3:]
-        tics = list(sorted(tz.unique(starts + ends)))
-        groups = tz.groupby(lambda d: pprint_task(d[1], dsk, label_size), results)
+        tics = sorted(unique(starts + ends))
+        groups = groupby(lambda d: pprint_task(d[1], dsk, label_size), results)
         data = {}
         for k, vals in groups.items():
             cnts = dict.fromkeys(tics, 0)
             for v in vals:
                 cnts[v.cache_time] += v.metric
                 cnts[v.free_time] -= v.metric
-            data[k] = [0] + list(tz.accumulate(add, tz.pluck(1, sorted(cnts.items()))))
+            data[k] = [0] + list(accumulate(add, pluck(1, sorted(cnts.items()))))
 
         tics = [0] + [i - start_time for i in tics]
         p = bp.figure(x_range=[0, max(tics)], **defaults)
