@@ -1,45 +1,46 @@
+import datetime
 import os
-import pytest
-from operator import add, mul
 import subprocess
 import sys
 import time
 from collections import OrderedDict
+from operator import add, mul
 
-from tlz import merge
+import pytest
+from tlz import compose, curry, merge, partial
 
 import dask
+import dask.bag as db
 from dask import delayed
 from dask.base import (
-    compute,
-    tokenize,
-    normalize_token,
-    normalize_function,
-    visualize,
-    persist,
-    function_cache,
-    is_dask_collection,
     DaskMethodsMixin,
-    optimize,
-    unpack_collections,
-    named_schedulers,
-    get_scheduler,
+    clone_key,
     collections_to_dsk,
+    compute,
+    compute_as_if_collection,
+    function_cache,
     get_collection_names,
     get_name_from_key,
+    get_scheduler,
+    is_dask_collection,
+    named_schedulers,
+    normalize_function,
+    normalize_token,
+    optimize,
+    persist,
     replace_name_in_key,
-    clone_key,
+    tokenize,
+    unpack_collections,
+    visualize,
 )
 from dask.core import literal
 from dask.delayed import Delayed
-from dask.utils import tmpdir, tmpfile
-from dask.utils_test import dec, inc, import_or_none
 from dask.diagnostics import Profiler
+from dask.highlevelgraph import HighLevelGraph
+from dask.utils import tmpdir, tmpfile
+from dask.utils_test import dec, import_or_none, inc
 
-
-tz = pytest.importorskip("tlz")
 da = import_or_none("dask.array")
-db = import_or_none("dask.bag")
 dd = import_or_none("dask.dataframe")
 np = import_or_none("numpy")
 sp = import_or_none("scipy.sparse")
@@ -59,39 +60,24 @@ def f3(a):
 
 
 def test_normalize_function():
-
     assert normalize_function(f2)
 
     assert normalize_function(lambda a: a)
 
-    assert normalize_function(tz.partial(f2, b=2)) == normalize_function(
-        tz.partial(f2, b=2)
-    )
+    assert normalize_function(partial(f2, b=2)) == normalize_function(partial(f2, b=2))
 
-    assert normalize_function(tz.partial(f2, b=2)) != normalize_function(
-        tz.partial(f2, b=3)
-    )
+    assert normalize_function(partial(f2, b=2)) != normalize_function(partial(f2, b=3))
 
-    assert normalize_function(tz.partial(f1, b=2)) != normalize_function(
-        tz.partial(f2, b=2)
-    )
+    assert normalize_function(partial(f1, b=2)) != normalize_function(partial(f2, b=2))
 
-    assert normalize_function(tz.compose(f2, f3)) == normalize_function(
-        tz.compose(f2, f3)
-    )
+    assert normalize_function(compose(f2, f3)) == normalize_function(compose(f2, f3))
 
-    assert normalize_function(tz.compose(f2, f3)) != normalize_function(
-        tz.compose(f2, f1)
-    )
+    assert normalize_function(compose(f2, f3)) != normalize_function(compose(f2, f1))
 
-    assert normalize_function(tz.curry(f2)) == normalize_function(tz.curry(f2))
-    assert normalize_function(tz.curry(f2)) != normalize_function(tz.curry(f1))
-    assert normalize_function(tz.curry(f2, b=1)) == normalize_function(
-        tz.curry(f2, b=1)
-    )
-    assert normalize_function(tz.curry(f2, b=1)) != normalize_function(
-        tz.curry(f2, b=2)
-    )
+    assert normalize_function(curry(f2)) == normalize_function(curry(f2))
+    assert normalize_function(curry(f2)) != normalize_function(curry(f1))
+    assert normalize_function(curry(f2, b=1)) == normalize_function(curry(f2, b=1))
+    assert normalize_function(curry(f2, b=1)) != normalize_function(curry(f2, b=2))
 
 
 def test_tokenize():
@@ -225,7 +211,7 @@ def test_tokenize_numpy_ufunc_consistent():
 
 
 def test_tokenize_partial_func_args_kwargs_consistent():
-    f = tz.partial(f3, f2, c=f1)
+    f = partial(f3, f2, c=f1)
     res = normalize_token(f)
     sol = (
         b"cdask.tests.test_base\nf3\np0\n.",
@@ -236,7 +222,7 @@ def test_tokenize_partial_func_args_kwargs_consistent():
 
 
 def test_normalize_base():
-    for i in [1, 1.1, "1", slice(1, 2, 3)]:
+    for i in [1, 1.1, "1", slice(1, 2, 3), datetime.date(2021, 6, 25)]:
         assert normalize_token(i) is i
 
 
@@ -299,13 +285,12 @@ def test_tokenize_pandas_extension_array():
         ),
     ]
 
-    if dd._compat.PANDAS_GT_100:
-        arrays.extend(
-            [
-                pd.array(["a", "b", None], dtype="string"),
-                pd.array([True, False, None], dtype="boolean"),
-            ]
-        )
+    arrays.extend(
+        [
+            pd.array(["a", "b", None], dtype="string"),
+            pd.array([True, False, None], dtype="boolean"),
+        ]
+    )
 
     for arr in arrays:
         assert tokenize(arr) == tokenize(arr)
@@ -461,6 +446,17 @@ def test_tokenize_object_with_recursion_error_returns_uuid():
     assert len(tokenize(cycle)) == 32
 
 
+def test_tokenize_datetime_date():
+    # Same date
+    assert tokenize(datetime.date(2021, 6, 25)) == tokenize(datetime.date(2021, 6, 25))
+    # Different year
+    assert tokenize(datetime.date(2021, 6, 25)) != tokenize(datetime.date(2022, 6, 25))
+    # Different month
+    assert tokenize(datetime.date(2021, 6, 25)) != tokenize(datetime.date(2021, 7, 25))
+    # Different day
+    assert tokenize(datetime.date(2021, 6, 25)) != tokenize(datetime.date(2021, 6, 26))
+
+
 def test_is_dask_collection():
     class DummyCollection:
         def __init__(self, dsk):
@@ -741,7 +737,6 @@ def test_custom_collection():
     assert z3.compute() == (70,)
 
 
-@pytest.mark.skipif("not db")
 def test_compute_no_opt():
     # Bag does `fuse` by default. Test that with `optimize_graph=False` that
     # doesn't get called. We check this by using a callback to track the keys
@@ -749,8 +744,8 @@ def test_compute_no_opt():
     from dask.callbacks import Callback
 
     b = db.from_sequence(range(100), npartitions=4)
-    add1 = tz.partial(add, 1)
-    mul2 = tz.partial(mul, 2)
+    add1 = partial(add, 1)
+    mul2 = partial(mul, 2)
     o = b.map(add1).map(mul2)
     # Check that with the kwarg, the optimization doesn't happen
     keys = []
@@ -908,7 +903,7 @@ def test_compute_dataframe_invalid_unicode():
     dd.from_pandas(df, npartitions=4)
 
 
-@pytest.mark.skipif("not da or not db")
+@pytest.mark.skipif("not da")
 def test_compute_array_bag():
     x = da.arange(5, chunks=2)
     b = db.from_sequence([1, 2, 3])
@@ -972,6 +967,23 @@ def test_visualize():
         visualize(x, filename=os.path.join(d, "mydask.png"))
         assert os.path.exists(os.path.join(d, "mydask.png"))
 
+        # To see if visualize() works when the filename parameter is set to None
+        # If the function raises an error, the test will fail
+        x.visualize(filename=None)
+
+
+@pytest.mark.skipif("not da")
+@pytest.mark.skipif(
+    sys.flags.optimize, reason="graphviz exception with Python -OO flag"
+)
+def test_visualize_highlevelgraph():
+    graphviz = pytest.importorskip("graphviz")
+    with tmpdir() as d:
+        x = da.arange(5, chunks=2)
+        viz = x.dask.visualize(filename=os.path.join(d, "mydask.png"))
+        # check visualization will automatically render in the jupyter notebook
+        assert isinstance(viz, graphviz.dot.Digraph)
+
 
 @pytest.mark.skipif(
     sys.flags.optimize, reason="graphviz exception with Python -OO flag"
@@ -1001,7 +1013,6 @@ def test_visualize_order():
 
 
 def test_use_cloudpickle_to_tokenize_functions_in__main__():
-    pytest.importorskip("cloudpickle")
     from textwrap import dedent
 
     defn = dedent(
@@ -1197,7 +1208,7 @@ def test_persist_delayedattr():
     assert xx.compute() == 1
 
 
-@pytest.mark.skipif("not da or not db")
+@pytest.mark.skipif("not da")
 def test_persist_array_bag():
     x = da.arange(5, chunks=2) + 1
     b = db.from_sequence([1, 2, 3]).map(inc)
@@ -1219,7 +1230,6 @@ def test_persist_array_bag():
     assert list(b) == list(bb)
 
 
-@pytest.mark.skipif("not db")
 def test_persist_bag():
     a = db.from_sequence([1, 2, 3], npartitions=2).map(lambda x: x * 2)
     assert len(a.__dask_graph__()) == 4
@@ -1229,7 +1239,6 @@ def test_persist_bag():
     db.utils.assert_eq(a, b)
 
 
-@pytest.mark.skipif("not db")
 def test_persist_item():
     a = db.from_sequence([1, 2, 3], npartitions=2).map(lambda x: x * 2).min()
     assert len(a.__dask_graph__()) == 7
@@ -1239,7 +1248,6 @@ def test_persist_item():
     db.utils.assert_eq(a, b)
 
 
-@pytest.mark.skipif("not db")
 def test_persist_bag_rename():
     a = db.from_sequence([1, 2, 3], npartitions=2)
     rebuild, args = a.__dask_postpersist__()
@@ -1251,7 +1259,6 @@ def test_persist_bag_rename():
     db.utils.assert_eq(b, [4, 5, 6])
 
 
-@pytest.mark.skipif("not db")
 def test_persist_item_change_name():
     a = db.from_sequence([1, 2, 3]).min()
     rebuild, args = a.__dask_postpersist__()
@@ -1270,7 +1277,6 @@ def test_normalize_function_limited_size():
 
 def test_optimize_globals():
     da = pytest.importorskip("dask.array")
-    db = pytest.importorskip("dask.bag")
 
     x = da.ones(10, chunks=(5,))
 
@@ -1340,10 +1346,12 @@ def test_raise_get_keyword():
 
 def test_get_scheduler():
     assert get_scheduler() is None
+    assert get_scheduler(scheduler=dask.local.get_sync) is dask.local.get_sync
     assert get_scheduler(scheduler="threads") is dask.threaded.get
     assert get_scheduler(scheduler="sync") is dask.local.get_sync
+    assert callable(get_scheduler(scheduler=dask.local.synchronous_executor))
     with dask.config.set(scheduler="threads"):
-        assert get_scheduler(scheduler="threads") is dask.threaded.get
+        assert get_scheduler() is dask.threaded.get
     assert get_scheduler() is None
 
 
@@ -1362,15 +1370,14 @@ def test_callable_scheduler():
 @pytest.mark.slow
 @pytest.mark.parametrize("scheduler", ["threads", "processes"])
 def test_num_workers_config(scheduler):
-    pytest.importorskip("cloudpickle")
     # Regression test for issue #4082
 
     f = delayed(pure=False)(time.sleep)
     # Be generous with the initial sleep times, as process have been observed
     # to take >0.5s to spin up
-    a = [f(1.0), f(1.0), f(1.0), f(0.1)]
     num_workers = 3
-    with dask.config.set(num_workers=num_workers), Profiler() as prof:
+    a = [f(1.0) for i in range(num_workers)]
+    with dask.config.set(num_workers=num_workers, chunksize=1), Profiler() as prof:
         compute(*a, scheduler=scheduler)
 
     workers = {i.worker_id for i in prof.results}
@@ -1400,3 +1407,29 @@ def test_clone_key():
     )
     with pytest.raises(TypeError):
         clone_key(1, 2)
+
+
+def test_compte_as_if_collection_low_level_task_graph():
+    # See https://github.com/dask/dask/pull/7969
+    da = pytest.importorskip("dask.array")
+    x = da.arange(10)
+
+    # Boolean flag to ensure MyDaskArray.__dask_optimize__ is called
+    optimized = False
+
+    class MyDaskArray(da.Array):
+        """Dask Array subclass with validation logic in __dask_optimize__"""
+
+        @classmethod
+        def __dask_optimize__(cls, dsk, keys, **kwargs):
+            # Ensure `compute_as_if_collection` don't convert to a low-level task graph
+            assert type(dsk) is HighLevelGraph
+            nonlocal optimized
+            optimized = True
+            return super().__dask_optimize__(dsk, keys, **kwargs)
+
+    result = compute_as_if_collection(
+        MyDaskArray, x.__dask_graph__(), x.__dask_keys__()
+    )[0]
+    assert optimized
+    da.utils.assert_eq(x, result)
