@@ -1,16 +1,18 @@
-from collections import namedtuple
-from operator import add, setitem
-from functools import partial
 import pickle
-from random import random
 import types
+from collections import namedtuple
+from functools import partial
+from operator import add, setitem
+from random import random
 
-from tlz import merge
+import cloudpickle
 import pytest
+from tlz import merge
 
 import dask
+import dask.bag as db
 from dask import compute
-from dask.delayed import delayed, to_task_dask, Delayed
+from dask.delayed import Delayed, delayed, to_task_dask
 from dask.utils_test import inc
 
 try:
@@ -98,7 +100,9 @@ def test_delayed_with_dataclass():
     dataclasses = pytest.importorskip("dataclasses")
 
     # Avoid @dataclass decorator as Python < 3.7 fail to interpret the type hints
-    ADataClass = dataclasses.make_dataclass("ADataClass", [("a", int)])
+    ADataClass = dataclasses.make_dataclass(
+        "ADataClass", [("a", int), ("b", int, dataclasses.field(init=False))]
+    )
 
     literal = dask.delayed(3)
     with_class = dask.delayed({"a": ADataClass(a=literal)})
@@ -393,6 +397,17 @@ def test_nout():
     assert x.compute() == tuple()
 
 
+@pytest.mark.parametrize(
+    "x",
+    [[1, 2], (1, 2), (add, 1, 2), [], ()],
+)
+def test_nout_with_tasks(x):
+    length = len(x)
+    d = delayed(x, nout=length)
+    assert len(d) == len(list(d)) == length
+    assert d.compute() == x
+
+
 def test_kwargs():
     def mysum(a, b, c=(), **kwargs):
         return a + b + sum(c) + sum(kwargs.values())
@@ -442,7 +457,6 @@ def test_array_delayed():
 
 
 def test_array_bag_delayed():
-    db = pytest.importorskip("dask.bag")
     da = pytest.importorskip("dask.array")
     np = pytest.importorskip("numpy")
 
@@ -560,7 +574,8 @@ def test_finalize_name():
     def key(s):
         if isinstance(s, tuple):
             s = s[0]
-        return s.split("-")[0]
+        # Ignore _ in 'ones_like'
+        return s.split("-")[0].replace("_", "")
 
     assert all(key(k).isalpha() for k in v.dask)
 
@@ -671,7 +686,6 @@ def test_pickle(f):
     "f", [delayed(modlevel_eager), modlevel_delayed1, modlevel_delayed2]
 )
 def test_cloudpickle(f):
-    cloudpickle = pytest.importorskip("cloudpickle")
     d = f(2)
     d = cloudpickle.loads(cloudpickle.dumps(d, protocol=pickle.HIGHEST_PROTOCOL))
     assert d.compute() == 3
