@@ -4,16 +4,17 @@ A threaded shared-memory scheduler
 See local.py
 """
 import atexit
+import multiprocessing.pool
 import sys
-from collections import defaultdict
-from multiprocessing.pool import ThreadPool
 import threading
-from threading import current_thread, Lock
+from collections import defaultdict
+from concurrent.futures import ThreadPoolExecutor
+from threading import Lock, current_thread
 
 from . import config
+from .local import MultiprocessingPoolExecutor, get_async
 from .system import CPU_COUNT
-from .local import get_async
-from .utils_test import inc, add  # noqa: F401
+from .utils_test import add, inc  # noqa: F401
 
 
 def _thread_get_id():
@@ -63,19 +64,21 @@ def get(dsk, result, cache=None, num_workers=None, pool=None, **kwargs):
         if pool is None:
             if num_workers is None and thread is main_thread:
                 if default_pool is None:
-                    default_pool = ThreadPool(CPU_COUNT)
-                    atexit.register(default_pool.close)
+                    default_pool = ThreadPoolExecutor(CPU_COUNT)
+                    atexit.register(default_pool.shutdown)
                 pool = default_pool
             elif thread in pools and num_workers in pools[thread]:
                 pool = pools[thread][num_workers]
             else:
-                pool = ThreadPool(num_workers)
-                atexit.register(pool.close)
+                pool = ThreadPoolExecutor(num_workers)
+                atexit.register(pool.shutdown)
                 pools[thread][num_workers] = pool
+        elif isinstance(pool, multiprocessing.pool.Pool):
+            pool = MultiprocessingPoolExecutor(pool)
 
     results = get_async(
-        pool.apply_async,
-        len(pool._pool),
+        pool.submit,
+        pool._max_workers,
         dsk,
         result,
         cache=cache,
@@ -91,6 +94,6 @@ def get(dsk, result, cache=None, num_workers=None, pool=None, **kwargs):
             for t in list(pools):
                 if t not in active_threads:
                     for p in pools.pop(t).values():
-                        p.close()
+                        p.shutdown()
 
     return results
